<?php

namespace Litecms\Legal\Http\Controllers;

use Exception;
use Litecms\Legal\Actions\LegalCaseWorkflow;
use Litecms\Legal\Http\Requests\LegalCaseWorkflowRequest;
use Litecms\Legal\Http\Resources\LegalCaseResource;
use Litecms\Legal\Models\LegalCase;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class LegalCaseWorkflowController extends BaseController
{
    /**
     * Action controller function for legal_case.
     *
     * @param Model $legal_case
     * @param action next action for the legal_case.
     *
     * @return Response
     */
    public function __invoke(LegalCaseWorkflowRequest $request, LegalCase $legal_case, $trasition)
    {
        try {
            $request = $request->all();
            $legal_case = LegalCaseWorkflow::run($trasition, $legal_case, $request);
            $data = new LegalCaseResource($legal_case);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('legal::legal_case.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('legal/legal_case/' . $legal_case->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('legal/legal_case/' . $legal_case->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the legal_case and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $legal_case
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $legal_case, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $legal_case = LegalCase::findBySignedId($legal_case);
        $transitions = $this->transitions($legal_case, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('legal::legal_case.name') . ' ' . trans('Approvals'))
            ->view('legal::public.legal_case.workflow')
            ->layout('mobile')
            ->data(compact('legal_case', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the legal_case.
     *
     * @param PublicRequest $request
     * @param string $legal_case
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $legal_case, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $legal_case = LegalCase::findBySignedId($legal_case);
        $user = user();
        $legal_case = LegalCaseWorkflow::run($transition, $legal_case, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($legal_case, $user)
    {
        $transitions = [];
        foreach ($legal_case->workflow()->transitions($legal_case) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.legal_case.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'legal_case' => $legal_case->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $legal_case->workflow()->form($value);
            $array['label'] = trans('legal::legal_case.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
